<?php

/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

// Include Zend Escaper for HTML Output Encoding
require_once(realpath(__DIR__ . '/Component_ZendEscaper/Escaper.php'));
$escaper = new Zend\Escaper\Escaper('utf-8');

/*************************************
 * FUNCTION: SIMPLERISK HEALTH CHECK *
 *************************************/
function simplerisk_health_check()
{
	//$current_app_version = current_version("app");
	//$latest_app_version = latest_version("app");
	//$current_db_version = current_version("db");
	//$latest_db_version = latest_version("db");

	//echo "<b><u>SimpleRisk Versions</u></b><br />";

	// Check that we are running the latest version of the SimpleRisk application
	//check_app_version($current_app_version, $latest_app_version);

	// Check that we are running the latest version of the SimpleRisk database
	//check_db_version($current_db_version, $latest_db_version);

	// Check that the application and database versions are the same
	//check_same_app_and_db($current_app_version, $current_db_version);

	echo "<br /><b><u>Connectivity</u></b><br />";

        // Check the SimpleRisk database connectivity
        check_database_connectivity();

        // Check that SimpleRisk can communicate with the API
        check_api_connectivity();

        // Check that SimpleRisk can connect to the services platform
        check_web_connectivity();

	echo "<br /><b><u>PHP</u></b><br />";

	// Check that this is PHP 7
	check_php_version();

        // Check the PHP memory_limit
        check_php_memory_limit();

        // Check the PHP max_input_vars
        check_php_max_input_vars();
	
	// Check the necessary PHP extensions are installed
	check_php_extensions();

	echo "<br /><b><u>MySQL</u></b><br />";

	// Check if MySQL STRICT SQL mode is enabled
	check_strict_sql_mode();

	echo "<br /><b><u>File and Directory Permissions</u></b><br />";

	// Check the simplerisk directory permissions
	check_simplerisk_directory_permissions();
}

/*******************************
 * FUNCTION: HEALTH CHECK GOOD *
 *******************************/
function health_check_good($text)
{
	global $escaper;

	echo "<img src=\"../images/check-mark-8-16.png\" />&nbsp&nbsp;" . $escaper->escapeHtml($text) . "<br />";
}

/******************************
 * FUNCTION: HEALTH CHECK BAD *
 ******************************/
function health_check_bad($text)
{
	global $escaper;

        echo "<img src=\"../images/x-mark-5-16.png\" />&nbsp;&nbsp;" . $escaper->escapeHtml($text) . "<br />";
}

/*******************************
 * FUNCTION: CHECK APP VERSION *
 *******************************/
function check_app_version($current_app_version, $latest_app_version)
{
	// If the current and latest versions are the same
        if ($current_app_version === $latest_app_version)
        {
                health_check_good("Running the current version (" . $current_app_version . ") of the SimpleRisk application.");
        }
        else
        {
                health_check_bad("Running an outdated version (" . $current_app_version . ") of the SimpleRisk application.");
        }
}

/******************************
 * FUNCTION: CHECK DB VERSION *
 ******************************/
function check_db_version($current_db_version, $latest_db_version)
{
	// If the current and latest versions are the same
        if ($current_db_version === $latest_db_version)
        {
                health_check_good("Running the current version (" . $current_db_version . ") of the SimpleRisk database schema.");
        }
        else
        {
                health_check_bad("Running an outdated version (" . $current_db_version . ") of the SimpleRisk database schema.");
        }
}

/***********************************
 * FUNCTION: CHECK SAME APP AND DB *
 ***********************************/
function check_same_app_and_db($current_app_version, $current_db_version)
{
	// If the current versions of the app and db are the same
	if ($current_app_version === $current_db_version)
	{
		health_check_good("The SimpleRisk application and database are at the same versions (" . $current_db_version . ").");
	}
	else
	{
		health_check_bad("The SimpleRisk application (" . $current_app_version . ") and database (" . $current_db_version . ") versions are not the same.");
	}
}

/****************************************************
 * FUNCTION: CHECK SIMPLERISK DIRECTORY PERMISSIONS *
 ****************************************************/
function check_simplerisk_directory_permissions()
{
	$simplerisk_dir = realpath(__DIR__ . '/..');

	// If the simplerisk directory is writeable
	if (is_writeable($simplerisk_dir))
	{
		health_check_good("The SimpleRisk directory (" . $simplerisk_dir . ") is writeable by the web user.");
	}
	else
	{
		health_check_bad("The SimpleRisk directory (" . $simplerisk_dir . ") is not writeable by the web user.");
	}

	$objects = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($simplerisk_dir), RecursiveIteratorIterator::SELF_FIRST);

	foreach ($objects as $name => $object)
	{
		// Do not check the directory above the SimpleRisk directory
		if ($name != $simplerisk_dir . "/..")
		{
			// If the directory is writeable
			if (!is_writeable($name))
			{
				health_check_bad($name . " is not writeable by the web user.");
			}
		}
	}
}

/************************************
 * FUNCTION: CHECK WEB CONNECTIVITY *
 ************************************/
function check_web_connectivity()
{
        // Configure the proxy server if one exists
        $method = "GET";
        $header = "content-type: Content-Type: application/x-www-form-urlencoded";

	// Set the stream context
	$http_context = array();
	$http_context['method'] = $method;
	$http_context['header'] = $header;
	$stream_context = array ('http' => $http_context);
	stream_context_set_default($stream_context);

	// URLs to check
	$urls = array("https://register.simplerisk.com", "https://services.simplerisk.com", "https://updates.simplerisk.com", "https://olbat.github.io", "https://github.com", "https://raw.githubusercontent.com");

	// Check the URLs
	foreach ($urls as $url)
	{
		// Get the headers for the URL
		$file_headers = @get_headers($url, 1);

		if(!$file_headers || $file_headers[0] == 'HTTP/1.1 404 Not Found')
		{
			health_check_bad("SimpleRisk was unable to connect to " . $url . ".");
		}
		else
		{
			health_check_good("SimpleRisk connected to " . $url . ".");
		}
	}
}

/**********************************
 * FUNCTION: CHECK PHP EXTENSIONS *
 **********************************/
function check_php_extensions()
{
	// List of extensions to check for
	$extensions = array("pdo", "pdo_mysql", "json", "phar", "zlib", "mbstring", "ldap", "dom", "curl", "posix", "zip", "gd");

	// For each extension
	foreach ($extensions as $extension)
	{
		if (extension_loaded($extension))
		{
			health_check_good("The PHP \"" . $extension . "\" extension is loaded.");
		}
		else
		{
			health_check_bad("The PHP \"" . $extension . "\" extension is not loaded.");
		}
	}
}

/*****************************************
 * FUNCTION: CHECK DATABASE CONNECTIVITY *
 *****************************************/
function check_database_connectivity()
{
	// Try opening a database connection
	$db = db_open();
	if ($db !== null)
	{
		// Close the database connection
		db_close($db);

		health_check_good("Communicated with the SimpleRisk database successfully.");
	}
	else
	{
		health_check_bad("Unable to communicate with the SimpleRisk database.");
	}
}

/************************************
 * FUNCTION: CHECK API CONNECTIVITY *
 ************************************/
function check_api_connectivity()
{
	// Get the SimpleRisk base URL
	$base_url = $_SESSION['base_url'];

	// Create the whoami URL
	$url = $base_url . "/api/whoami";

	// Test the API URL
	$headers = get_headers($url);
	$code = substr($headers[0], 9, 3);

	// If the response code is success or unauthorized
	if ($code == 200 || $code = 401)
	{
		health_check_good("Communicated with the SimpleRisk API successfully.");
	}
	else
	{
		health_check_bad("Unable to communicate with the SimpleRisk API.");
	}
}

/***********************************
 * FUNCTION: CHECK STRICT SQL MODE *
 ***********************************/
function check_strict_sql_mode()
{
        // Open a database connection
        $db = db_open();

	// Query for the current SQL mode
	$stmt = $db->prepare("SELECT @@sql_mode;");
	$stmt->execute();
	$array = $stmt->fetch();
	$sql_mode = $array['@@sql_mode'];

	// Close the database connection
	db_close($db);

	// If the row contains STRICT_TRANS_TABLES
	if (preg_match("/.*STRICT_TRANS_TABLES.*/", $sql_mode))
	{
		health_check_bad("SimpleRisk will not work properly with STRICT_TRANS_TABLES enabled.");
	}
	else
	{
		health_check_good("Verified that STRICT_TRANS_TABLES is not enabled for MySQL.");
	}
}

/*******************************
 * FUNCTION: CHECK PHP VERSION *
 *******************************/
function check_php_version()
{
	// Get the version of PHP
	if (!defined('PHP_VERSION_ID'))
	{
		$version = explode('.', PHP_VERSION);

		define('PHP_VERSION_ID', ($version[0] * 10000 + $version[1] * 100 + $version[2]));
	}

	// If PHP is at least 8.0
	if (PHP_VERSION_ID >= 80000)
	{
		health_check_bad("SimpleRisk has not been tested to work properly on PHP 8.0 or higher.");
		health_check_good("You are running a version of PHP that is currently supported by The PHP Group.  See https://www.php.net/supported-versions.php for more details.");
	}
	// If PHP is at least 7.3 but not higher than 8.0
	else if (PHP_VERSION_ID >= 70300 && PHP_VERSION_ID < 80000)
	{
		health_check_good("SimpleRisk has been tested to work properly on this version of PHP.");
		health_check_good("You are running a version of PHP that is currently supported by The PHP Group.  See https://www.php.net/supported-versions.php for more details.");
	}
	// If PHP is at least 7.1 but not higher than 7.3
	else if (PHP_VERSION_ID >= 70100 && PHP_VERSION_ID < 70300)
	{
		health_check_good("SimpleRisk has been tested to work properly on this version of PHP.");
		health_check_bad("PHP versions under 7.3 are no longer supported by The PHP Group.  See https://www.php.net/supported-versions.php for more details.");
	}
	// If this is less than PHP 7.1
	else
	{
		health_check_bad("SimpleRisk will not run properly under this version of PHP.  Please upgrade to PHP 7.1 or higher.");
		health_check_bad("PHP versions under 7.3 are no longer supported by The PHP Group.  See https://www.php.net/supported-versions.php for more details.");
	}
}

/************************************
 * FUNCTION: CHECK PHP MEMORY LIMIT *
 ************************************/
function check_php_memory_limit()
{
        // Get the currently set memory limit
        $memory_limit = ini_get('memory_limit');

        // If the memory limit is not set
        if ($memory_limit === false)
        {
		health_check_bad("No memory_limit value is set in the php.ini file and PHP is likely using the default value which is less than the current size of the SimpleRisk application.  SimpleRisk will function normally, however, this creates an issue with the one-click upgrade process.  We recommend setting the memory_limit value to 256M or higher.");
        }
        // If the memory limit is set to unlimited
        else if ($memory_limit == -1)
        {
		health_check_good("The memory_limit value in the php.ini file is set to -1.  This provides unlimited memory to PHP, which should be acceptable for the SimpleRisk application.");
        }
        // Otherwise
        else
        {
                // If the memory limit is a number followed by characters
                if (preg_match('/^(\d+)(.)$/', $memory_limit, $matches))
                {
                        // If the memory limit is in megabytes
                        if ($matches[2] == 'M')
                        {
                                // Get the memory limit in bytes
                                $memory_limit_bytes = $matches[1] * 1024 * 1024;
                        }
                        // If the memory limit is in kilobytes
                        else if ($matches[2] == 'K')
                        {
                                // Get the memory limit in bytes
                                $memory_limit_bytes = $matches[1] * 1024;
                        }
                }

                // Set the current SimpleRisk size in bytes
                $simplerisk_size_bytes = 180 * 1024 * 1024;

                // If the memory limit is less than the SimpleRisk size
                if ($memory_limit_bytes < $simplerisk_size_bytes)
                {
			health_check_bad("The memory_limit value in the php.ini file is set to {$memory_limit}, which is less than the current size of the SimpleRisk application.  SimpleRisk will function normally, however, this creates an issue with the one-click upgrade process.  We recommend setting the memory_limit value to 256M or higher.");
                }
                // The memory limit is higher than the SimpleRisk size
                else
                {
			health_check_good("The memory_limit value in the php.ini file is set to {$memory_limit}.");
                }
        }
}

/**************************************
 * FUNCTION: CHECK PHP MAX INPUT VARS *
 **************************************/
function check_php_max_input_vars()
{
        // Get the currently set max_iput_vars
        $max_input_vars = ini_get('max_input_vars');

        // If the max_input_vars is not set
        if ($max_input_vars === false)
        {
                return array("result" => 0, "text" => "The max_input_vars value in the php.ini file is not explicitly set.  The default value of 1000 is too low and the SimpleRisk Dynamic Risk Report will not function properly with this configuration.  We recommend setting the max_input_vars to 3000.");
        }
        // If the max_input_vars is set
        else
        {
                // If the max_input_vars is a number followed by characters
                if (preg_match('/^(\d+)$/', $max_input_vars, $matches))
                {
                        // If the max_input_vars is 1000
                        if ($max_input_vars == 1000)
                        {
				health_check_bad("The max_input_vars value in the php.ini file is set to the default value of 1000.  The SimpleRisk Dynamic Risk Report will not function properly with this configuration.  We recommend setting the max_input_vars to 3000.");
                        }
                        // If the max_input_vars is less than 3000
                        else if ($max_input_vars < 3000)
                        {
				health_check_bad("The max_input_vars value in the php.ini file is set to {$max_input_vars}, which could cause issues with the SimpleRisk Dynamic Risk Report.  We recommend setting the max_input_vars to 3000.");
                        }
                        // If the max_input_vars is 3000 or higher
                        else if ($max_input_vars >= 3000)
                        {
				health_check_good("The max_input_vars value in the php.ini file is set to {$max_input_vars}.");
                        }
                }
        }
}

?>
