<?php

// This is the release version we are running the database installer for
$release_version = "20211115-001";

?>

<html ng-app="SimpleRisk">
  <head>
    <title>SimpleRisk: Enterprise Risk Management Simplified</title>
      <link rel="stylesheet" type="text/css" href="css/bootstrap.min.css" media="screen" />
      <link rel="stylesheet" type="text/css" href="css/style.css" media="screen" />
      <link rel="stylesheet" href="css/bootstrap.css">
      <link rel="stylesheet" href="css/bootstrap-responsive.css">
      <link rel="stylesheet" href="css/font-awesome.min.css">
      <link rel="stylesheet" href="css/theme.css">
  </head>

  <body ng-controller="MainCtrl" class="login--page">

    <header class="l-header">
      <div class="navbar">
        <div class="navbar-inner">
          <div class="container-fluid">
            <a class="brand" href="https://www.simplerisk.com/"><img src="images/logo@2x.png" alt="SimpleRisk Logo" /></a>
            <div class="navbar-content pull-right">
              <ul class="nav">
                <li>
                  <a href="index.php">Database Installation Script</a>
                </li>
              </ul>
            </div>
          </div>
        </div>
      </div>
    </header>
    <div class="container-fluid">
      <div class="row-fluid">
        <div class="span12">
          <div class="login-wrapper clearfix">

<?php
// If we can load the SimpleRisk files
//if (functions_exists() && healthcheck_exists() && zendescaper_exists())
if (healthcheck_exists() && zendescaper_exists())
{
	// Include Zend Escaper for HTML Output Encoding
	$escaper = new Zend\Escaper\Escaper('utf-8');

	// If nothing has been posted yet
	if (!$_POST)
	{
		// Run the installer health check
		installer_health_check($release_version);
	}
	// If the user has selected to continue from the health check
	else if (isset($_POST) && isset($_POST['continue_from_health_check']))
	{
		// Run the installer database check
		installer_database_check();
	}
	// If the user has selected to validate their database credentials
	else if (isset($_POST) && isset($_POST['continue_from_database_check']))
	{
		// If the username posted is longer than 16 characters
		if (strlen($_POST['user']) > 16)
		{
			// Run the installer database check again
			installer_database_check();
		}
		// Otherwise, run the database validation
		else installer_validate_database();
	}
	// If the user has selected to start the install
	else if (isset($_POST) && isset($_POST['start_database_install']))
	{
                // If the username posted is longer than 16 characters
                if (strlen($_POST['sr_user']) > 16)
                {
                        // Run the database validation again
                        installer_validate_database();
                }
                // Otherwise, proceed with the installation
		else
		{
			// Check if the database already exists
			$database_exists = database_already_exists();

			// If the database already exists
			if ($database_exists != false)
			{
				// Run the database validation
				installer_validate_database($database_exists);
			}
			// Otherwise, the database doesn't already exist
			else
			{
				// Run the database installation
				installer_run_installation($release_version);
			}
		}
	}
	// If the user has selected to update the configuration
	else if (isset($_POST) && isset($_POST['update_config']))
	{
		// Update the SimpleRisk configuration
		installer_update_configuration();
	}
}
else
{
	echo "Couldn't find the SimpleRisk files under " . realpath(__DIR__ . "/../");
}

?>

          </div>
        </div>
      </div>
    </div>
  </body>

</html>

<?php

/******************************
 * FUNCTION: DATABASE CONNECT *
 ******************************/
function installer_db_open($dbhost, $dbport, $dbuser, $dbpass, $dbname)
{
        // Connect to the database
        try
        {
                $db = new PDO("mysql:charset=UTF8;dbname=".$dbname.";host=".$dbhost.";port=".$dbport,$dbuser,$dbpass, array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION));
                $db->setAttribute(PDO::MYSQL_ATTR_INIT_COMMAND, "SET NAMES utf8");
                $db->setAttribute(PDO::MYSQL_ATTR_INIT_COMMAND, "SET CHARACTER SET utf8");
		$db->setAttribute(PDO::MYSQL_ATTR_INIT_COMMAND, "SET SQL_MODE = 'NO_AUTO_VALUE_ON_ZERO'");

                return $db;
        }
        catch (PDOException $e)
        {
		die("Database Connection Failed: " . $e->getMessage());
                //printf("Unable to connect to the database.");
        }

        return null;
}

/*********************************
 * FUNCTION: DATABASE DISCONNECT *
 *********************************/
function installer_db_close($db)
{
        // Close the DB connection
        $db = null;
}

/******************************
 * FUNCTION: FUNCTIONS EXISTS *
 ******************************/
function functions_exists()
{
	// Path to the file
	$path = realpath(__DIR__ . "/../includes/functions.php");

	// If the SimpleRisk functions exist
	if (file_exists($path))
	{
        	// Load it
        	require_once($path);

		// Return true
		return true;
	}
	else return false;
}

/*******************************
 * FUNCTION: HEALTHCHECK EXIST *
 *******************************/
function healthcheck_exists()
{
	// Path to the file
	$path = realpath(__DIR__ . "/includes/healthcheck.php");

	// If the SimpleRisk healthcheck exists
	if (file_exists($path))
	{
        	// Load it
        	require_once($path);

		// Return true
		return true;
	}
	else return false;
}

/********************************
 * FUNCTION: ZENDESCAPER EXISTS *
 ********************************/
function zendescaper_exists()
{
	// Path to the file
	$path = realpath(__DIR__ . "/includes/Component_ZendEscaper/Escaper.php");

	// If the Zend Escaper exists
	if (file_exists($path))
	{
        	// Include Zend Escaper for HTML Output Encoding
        	require_once($path);

		// Return true
		return true;
	}
	else return false;
}

/************************************
 * FUNCTION: INSTALLER HEALTH CHECK *
 ************************************/
function installer_health_check($release_version)
{
	echo "<h1 class=\"text-center welcome--msg\">SimpleRisk Installer - Health Check</h1>\n";
	echo "<form name=\"continue\" method=\"post\" action=\"\" class=\"loginForm\">\n";

	echo "<b><u>PHP</u></b><br />\n";
	//check_installer_php_version();

        // Check that this is PHP 7
        check_php_version();

        // Check the PHP memory_limit
        check_php_memory_limit();

        // Check the PHP max_input_vars
        check_php_max_input_vars();

        // Check the necessary PHP extensions are installed
        check_php_extensions();

	//echo "<br /><b><u>SimpleRisk Version</u></b><br />\n";

	// Check that the installer version matches the application version
	//check_installer_application_version_match($release_version);

	echo "<br /><b><u>Connectivity</u></b><br />\n";

	// Check that SimpleRisk can communicate with the API
	//check_api_connectivity();

	// Check that SimpleRisk can connect to the services platform
	check_web_connectivity();

	echo "<br /><b><u>File and Directory Permissions</u></b><br />\n";

	// Check the simplerisk directory permissions
	check_simplerisk_directory_permissions();

	echo "<br />\n";
	echo "If everything looks alright with the health check above, click &quot;CONTINUE&quot; to proceed with the installation.<br /><br />\n";
	echo "<input type=\"submit\" name=\"continue_from_health_check\" value=\"CONTINUE\" />\n";
	echo "</form>\n";
}

/**************************************
 * FUNCTION: INSTALLER DATABASE CHECK *
 **************************************/
function installer_database_check($health_check_bad=null)
{
	global $escaper;

        echo "<h1 class=\"text-center welcome--msg\">SimpleRisk Installer - Database Check</h1>\n";
	echo "<form name=\"validate\" method=\"post\" action=\"\" class=\"loginForm\">\n";

	// If the health check came back
	if (!is_null($health_check_bad))
	{
		foreach ($health_check_bad as $value)
		{
			health_check_bad($value);
			echo "<br />\n";
		}
	}

        echo "Enter your database information to proceed with SimpleRisk install:<br /><br />\n";

        // Database connection information table
        echo "<table>\n";
        echo "<thead>\n";
        echo "<tr>\n";
        echo "<th align=\"left\" colspan=\"2\"><label class=\"login--label\">Database Connection Information</label></th>\n";
        echo "</tr>\n";
        echo "</thead>\n";
        echo "<tbody>\n";
        echo "<tr>\n";
        echo "<td><label for>Database IP/Host:&nbsp;&nbsp;</label></td>\n";
        echo "<td><input type=\"text\" size=\"30\" name=\"host\" value=\"" . (isset($_POST['host']) ? $escaper->escapeHtml($_POST['host']) : "localhost") . "\" /></td>\n";
        echo "</tr>\n";
        echo "<tr>\n";
        echo "<td><label for>Database Port:</label></td>\n";
        echo "<td><input type=\"text\" size=\"30\" name=\"port\" value=\"" . (isset($_POST['port']) ? $escaper->escapeHtml($_POST['port']) : "3306") . "\" /></td>\n";
        echo "</tr>\n";
        echo "<tr>\n";
        echo "<td><label for>Database User:</label></td>\n";
        echo "<td><input type=\"text\" size=\"30\" maxlength=\"16\" name=\"user\" id=\"user\" value=\"" . (isset($_POST['user']) ? $escaper->escapeHtml($_POST['user']) : "root") . "\" /></td>\n";
        echo "</tr>\n";
        echo "<tr>\n";
        echo "<td><label for>Database Pass:</label></td>\n";
        echo "<td><input type=\"password\" size=\"30\" name=\"pass\" value=\"" . (isset($_POST['pass']) ? $escaper->escapeHtml($_POST['pass']) : "") . "\" /></td>\n";
        echo "</tr>\n";
        echo "</tbody>\n";
        echo "</table>\n";
        echo "<br />\n";
	echo "<input type=\"submit\" name=\"continue_from_database_check\" value=\"VALIDATE DATABASE CREDENTIALS\" />\n";
	echo "</form>\n";
	echo "<script>\n";
	echo "function checkLength(value){\n";
	echo "  var maxLength = 16;\n";
	echo "  if (value.length >= maxLength) return false;\n";
	echo "  return true;\n";
	echo "}\n";
	echo "document.getElementById('user').onkeyup = function(){\n";
	echo "  if(!checkLength(this.value)) alert('MySQL usernames cannot be longer than 16 characters!');\n";
	echo "}\n";
	echo "</script>\n";
}

/*****************************************
 * FUNCTION: INSTALLER VALIDATE DATABASE *
 *****************************************/
function installer_validate_database($health_check = null)
{
        // Database Connection Information
        $dbhost = addslashes($_POST['host']);
        $dbport = addslashes($_POST['port']);
        $dbuser = addslashes($_POST['user']);
        $dbpass = addslashes($_POST['pass']);
	$dbname = "mysql";

        // Remove any backticks from DB connection information
        $pattern = '/`/';
        $replacement = '';
        $dbhost = preg_replace($pattern, $replacement, $dbhost);
        $dbport = preg_replace($pattern, $replacement, $dbport);
        $dbuser = preg_replace($pattern, $replacement, $dbuser);
        $dbpass = preg_replace($pattern, $replacement, $dbpass);

	// Successful connection
	$success = true;

        // Connect to the database
        try
        {
		$db = new PDO("mysql:charset=UTF8;dbname=".$dbname.";host=".$dbhost.";port=".$dbport,$dbuser,$dbpass, array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION));
        }
        catch (PDOException $e)
        {
		// Set success to false
		$success = false;
        }

	// Set the error to false and create an empty error array
	$error = false;
	$error_message = array();

	// If the connection was successful
	if ($success)
	{
		// Otherwise, we have a successful connection
        	echo "<h1 class=\"text-center welcome--msg\">SimpleRisk Installer - Database Install</h1>\n";
		echo "<form name=\"install\" method=\"post\" action=\"\" class=\"loginForm\">\n";

        	health_check_good("Successfully connected to the database.");

		// If the health check varibale isn't null
		if (!is_null($health_check))
		{
			health_check_bad($health_check);
		}

		// If STRICT mode is enabled
		if (check_mysql_strict())
		{
			$error_message[] = "SimpleRisk will not work properly with STRICT_TRANS_TABLES enabled.";
			$error = true;
		}
		else
		{
			health_check_good("Verified that STRICT_TRANS_TABLES is not enabled for MySQL.");
		}

		// If NO_ZERO_DATE is enabled
		if (check_mysql_no_zero_date())
		{
			$error_message[] = "SimpleRisk will not work properly with NO_ZERO_DATE enabled.";
			$error = true;
		}
		else
		{
			health_check_good("Verified that NO_ZERO_DATE is not enabled for MySQL.");
		}

		// If ONLY_FULL_GROUP_BY is enabled
		if (check_mysql_only_full_group_by())
		{
			$error_message[] = "SimpleRisk will not work properly with ONLY_FULL_GROUP_BY enabled.";
			$error = true;
		}
		else
		{
			health_check_good("Verified that ONLY_FULL_GROUP_BY is not enabled for MySQL.");
		}

		// If there is not an error
		if (!$error)
		{
			// Prompt for SimpleRisk installation information
			installer_simplerisk_database($dbhost, $dbport, $dbuser, $dbpass);
		}
		else
		{
			installer_database_check($error_message);
		}
	}
	else
	{
		$error_message = "Unable to connect to the database.";
		installer_database_check($error_message);
	}
}

/********************************************
 *  FUNCTION: INSTALLER SIMPLERISK DATABASE *
 ********************************************/
function installer_simplerisk_database($dbhost, $dbport, $dbuser, $dbpass)
{
	global $escaper;

        // Create a random password
        $generated_password = generate_token(20);

        echo "<br />Enter your database information to proceed with SimpleRisk install:<br /><br />\n";

	// Hidden fields for the working database credentials
        echo "<input type=\"hidden\" name=\"host\" value=\"" . $escaper->escapeHtml($dbhost) . "\" />\n";
        echo "<input type=\"hidden\" name=\"port\" value=\"" . $escaper->escapeHtml($dbport) . "\" />\n";
        echo "<input type=\"hidden\" name=\"user\" value=\"" . $escaper->escapeHtml($dbuser) . "\" />\n";
        echo "<input type=\"hidden\" name=\"pass\" value=\"" . $escaper->escapeHtml($dbpass) . "\" />\n";

        // SimpleRisk installation information table
        echo "<table>\n";
        echo "<thead>\n";
        echo "<tr>\n";
        echo "<th align=\"left\" colspan=\"3\"><label class=\"login--label\">SimpleRisk Installation Information</label></th>\n";
        echo "</tr>\n";
        echo "</thead>\n";
        echo "<tbody>\n";
        echo "<tr>\n";
        echo "<td><label for>SimpleRisk IP/Host:</label></td>\n";
        echo "<td><input type=\"text\" size=\"30\" name=\"sr_host\" value=\"" . (isset($_POST['sr_host']) ? $escaper->escapeHtml($_POST['sr_host']) : $escaper->escapeHtml($_POST['host'])) . "\" /></td>\n";
        echo "<td></td>\n";
        echo "</tr>\n";
        echo "<tr>\n";
        echo "<td><label for>SimpleRisk Database:</label></td>\n";
        echo "<td><input type=\"text\" size=\"30\" name=\"sr_db\" value=\"" . (isset($_POST['sr_db']) ? $escaper->escapeHtml($_POST['sr_db']) : "simplerisk") . "\" /></td>\n";
        echo "<td><label for>&nbsp;&nbsp;<== Default SimpleRisk Database Name</label></td>\n";
        echo "</tr>\n";
        echo "<tr>\n";
        echo "<td><label for>SimpleRisk User:</label></td>\n";
        echo "<td><input type=\"text\" size=\"30\" maxlength=\"16\" name=\"sr_user\" id=\"sr_user\" value=\"" . (isset($_POST['sr_user']) ? $escaper->escapeHtml($_POST['sr_user']) : "simplerisk") . "\" /></td>\n";
        echo "<td><label for>&nbsp;&nbsp;<== Default SimpleRisk Username (16 Characters Max)</label></td>\n";
        echo "</tr>\n";
        echo "<tr>\n";
        echo "<td><label for>SimpleRisk Pass:</label></td>\n";
        echo "<td><input type=\"password\" size=\"30\" name=\"sr_pass\" value=\"" . (isset($_POST['sr_pass']) ? $escaper->escapeHtml($_POST['sr_pass']) : $escaper->escapeHtml($generated_password)) . "\" /></td>\n";
        echo "<td><label for>&nbsp;&nbsp;<== Automatically Generated Random Password</label></td>\n";
        echo "</tr>\n";
        echo "</tbody>\n";
        echo "</table>\n";
        echo "<br />\n";
        echo "<script>\n";
        echo "function checkLength(value){\n";
        echo "  var maxLength = 16;\n";
        echo "  if (value.length >= maxLength) return false;\n";
        echo "  return true;\n";
        echo "}\n";
        echo "document.getElementById('sr_user').onkeyup = function(){\n";
        echo "  if(!checkLength(this.value)) alert('MySQL usernames cannot be longer than 16 characters!');\n";
        echo "}\n";
        echo "</script>\n";

        // SimpleRisk configuration information table
        echo "<table>\n";
        echo "<thead>\n";
        echo "<tr>\n";
        echo "<th align=\"left\" colspan=\"2\"><label class=\"login--label\">SimpleRisk Configuration Information</label></th>\n";
        echo "</tr>\n";
        echo "</thead>\n";
        echo "<tbody>\n";
        echo "<tr>\n";

        echo "<td><label for>SimpleRisk Base URL:</label></td>\n";
        echo "<td><input type=\"text\" size=\"30\" name=\"simplerisk_base_url\" value=\"" . $escaper->escapeHtml(get_current_url()) . "\" /></td>\n";
        echo "<td></td>\n";
        echo "</tr>\n";
        echo "<tr>\n";
        echo "<td><label for>Database Schema:</label></td>\n";
        echo "<td>\n";
        echo "<select name=\"schema\">\n";
        echo "<option value=\"1\"" . (isset($_POST['schema']) && $_POST['schema'] == 1 ? " selected" : "") . ">Default - English</option>\n";
        echo "<option value=\"2\"" . (isset($_POST['schema']) && $_POST['schema'] == 2 ? " selected" : "") . ">Default - Spanish</option>\n";
        echo "<option value=\"3\"" . (isset($_POST['schema']) && $_POST['schema'] == 3 ? " selected" : "") . ">Default - Brazilian Portuguese</option>\n";
        echo "</select>\n";
        echo "</td>\n";
        echo "</tr>\n";
        echo "<tr>\n";
        echo "<td><label for>Use Database for Sessions:</label></td>\n";
        echo "<td>\n";
        echo "<select name=\"db_sessions\">\n";

	// If a db_sessions value was posted
	if (isset($_POST) && isset($_POST['db_sessions']))
	{
		echo "<option value=\"true\"" . ($_POST['db_sessions'] == "true" ? " selected" : "") . ">true</option>\n";
		echo "<option value=\"false\"" . ($_POST['db_sessions'] == "false" ? " selected" : "") . ">false</option>\n";
	}
	else
	{
		echo "<option value=\"true\" selected>true</option>\n";
        	echo "<option value=\"false\">false</option>\n";
	}

        echo "</select><br />\n";
        echo "</td>\n";
        echo "</tr>\n";

        echo "</tbody>\n";
        echo "</table>\n";

        echo "<br /><input type=\"submit\" name=\"start_database_install\" value=\"INSTALL\" />\n";
        echo "</form>\n";
}

/****************************************
 * FUNCTION: INSTALLER RUN INSTALLATION *
 ****************************************/
function installer_run_installation($release_version)
{
	global $escaper;

        echo "<h1 class=\"text-center welcome--msg\">SimpleRisk Installer - Configuration Update</h1>\n";
	echo "<form name=\"update_config\" method=\"post\" action=\"\" class=\"loginForm\">\n";

        // Database Connection Information
        $dbhost = addslashes($_POST['host']);
        $dbport = addslashes($_POST['port']);
        $dbuser = addslashes($_POST['user']);
        $dbpass = addslashes($_POST['pass']);

	// Remove any backticks from DB connection information
	$pattern = '/`/';
	$replacement = '';
	$dbhost = preg_replace($pattern, $replacement, $dbhost);
	$dbport = preg_replace($pattern, $replacement, $dbport);
	$dbuser = preg_replace($pattern, $replacement, $dbuser);
	$dbpass = preg_replace($pattern, $replacement, $dbpass);

        // Connect to the mysql database
        $db = installer_db_open($dbhost, $dbport, $dbuser, $dbpass, "mysql");

        // SimpleRisk Installation Information
        $sr_host = addslashes($_POST['sr_host']);
        $sr_db = addslashes($_POST['sr_db']);
        $sr_user = addslashes($_POST['sr_user']);
        $sr_pass = addslashes($_POST['sr_pass']);
        $db_sessions = ((isset($_POST['db_sessions']) && $_POST['db_sessions'] == "false") ? "false" : "true");
	$db_ssl_cert_path = ((isset($_POST['db_ssl_cert_path']) && $_POST['db_ssl_cert_path'] != "") ? $escaper->escapeHtml($_POST['db_ssl_cert_path']) : "");
	$simplerisk_base_url = $_POST['simplerisk_base_url'];

	// Remove any backticks from DB connection information
	$pattern = '/`/';
	$replacement = '';
	$sr_host = preg_replace($pattern, $replacement, $sr_host);
	$sr_db = preg_replace($pattern, $replacement, $sr_db);
	$sr_user = preg_replace($pattern, $replacement, $sr_user);
	$sr_pass = preg_replace($pattern, $replacement, $sr_pass);

	// Create the grantee
	$grantee = "'" . $dbuser . "'@'" . $dbhost . "'";

	// Check if the grantees privileges include Super
	$stmt = $db->prepare("SELECT * FROM `information_schema`.`USER_PRIVILEGES` WHERE `GRANTEE`=:grantee AND `PRIVILEGE_TYPE`='SUPER';");
	$stmt->bindParam(":grantee", $grantee, PDO::PARAM_STR);
	$stmt->execute();
	$privileges = $stmt->fetchAll();

	// If the grantee has the super privilege
	if (count($privileges) > 0)
	{
                // Disable STRICT_TRANS_TABLES sql_mode
                $stmt = $db->prepare("SET GLOBAL sql_mode=(SELECT REPLACE(@@sql_mode,'STRICT_TRANS_TABLES',''));");
                $stmt->execute();

                // Disable NO_ZERO_DATE sql_mode
                $stmt = $db->prepare("SET GLOBAL sql_mode=(SELECT REPLACE(@@sql_mode,'NO_ZERO_DATE',''));");
                $stmt->execute();

		// Disable ONLY_FULL_GROUP_BY sql_mode
		$stmt = $db->prepare("SET GLOBAL sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));");
		$stmt->execute();
	}

        // Create the SimpleRisk database
        $stmt = $db->prepare("CREATE DATABASE `" . $sr_db . "`");
        $stmt->execute();

        // Create the SimpleRisk user
        $stmt = $db->prepare("CREATE USER '" . $sr_user . "'@'" . $sr_host . "' IDENTIFIED BY '" . $sr_pass . "'");
        $stmt->execute();

        // Grant the SimpleRisk user permissions
        $stmt = $db->prepare("GRANT SELECT,INSERT,UPDATE,ALTER,DELETE,CREATE,DROP,INDEX,REFERENCES ON `" . $sr_db . "`.* TO '" . $sr_user . "'@'" . $sr_host . "'");
        $stmt->execute();

        // Reload the privileges
        $stmt = $db->prepare("FLUSH PRIVILEGES");
        $stmt->execute();

        // Close the mysql database
        installer_db_close($db);

        // Get the schema
        $schema = $_POST['schema'];

        // Depending on the schema selected
        switch ($schema)
        {
                // Default - English
                case 1:
                        $file = "simplerisk-en-" . $release_version . ".sql";
                        load_file($dbhost, $dbport, $dbuser, $dbpass, $sr_db, $file);
                        break;
                // Default - Spanish
                case 2:
                        $file = "simplerisk-es-" . $release_version . ".sql";
                        load_file($dbhost, $dbport, $dbuser, $dbpass, $sr_db, $file);
                        break;
                // Default - Brazilian Portuguese
                case 3:
                        $file = "simplerisk-bp-" . $release_version . ".sql";
                        load_file($dbhost, $dbport, $dbuser, $dbpass, $sr_db, $file);
                        break;
                // No actual schema submitted   
                default:
                        echo "An invalid database schema was specified.<br /><br />\n";
                        break;
        }

	// Create a unique salt
	$salt = "";
	$values = array_merge(range(0, 9), range('a', 'z'), range('A', 'Z'));
	for ($i = 0; $i < 20; $i++)
	{
		$salt .= $values[array_rand($values)];
	}

	// Hash the salt
	$salt_hash = '$2a$15$' . md5($salt);

	// Generate the password hash for admin user
	set_time_limit(120);
	$hash = crypt("admin", $salt_hash);

	// Update the salt and hash for the admin user
	$stmt = $db->prepare("UPDATE `" . $sr_db . "`.`user` SET salt=:salt, password=:password WHERE value=1;");
	$stmt->bindParam(":salt", $salt, PDO::PARAM_STR);
	$stmt->bindParam(":password", $hash, PDO::PARAM_STR);
        $stmt->execute();

	// Update the simplerisk base url
	$stmt = $db->prepare("INSERT INTO `" . $sr_db . "`.`settings` (`name`,`value`) VALUES ('simplerisk_base_url', :simplerisk_base_url)");
	$stmt->bindParam(":simplerisk_base_url", $simplerisk_base_url, PDO::PARAM_STR);
	$stmt->execute();

        // This should be the path to the config.php file
        $file = realpath(__DIR__ . '/../includes/config.php');

        $config_file = create_config_file($dbhost, $dbport, $sr_user, $sr_pass, $sr_db, $schema, $db_sessions, $db_ssl_cert_path);

        // If the config.php file exists where we expect it
        if (file_exists($file))
        {
                // If the config.php file is writable
                if (is_writable($file))
                {
                        // Write out a temporary config.php file
                        $tmp_dir = sys_get_temp_dir();
                        $tmp_config_file = $tmp_dir . '/config.php';
                        $fh = fopen($tmp_config_file, 'w');
                        fwrite($fh, $config_file);
                        fclose($fh);

                        echo "Found a config.php file located at " . $file . ".  Should I update it with the configuration information below?<br /><br />\n";
                        echo "<input type=\"submit\" name=\"update_config\" value=\"UPDATE\" />\n";
                }
                else
                {
                        echo "Found a config.php file located at " . $file . " but it is not writeable.  Replace the file with the content below to get SimpleRisk to work properly.<br /><br />\n";
                        echo "<b>For security reasons, it is <u>HIGHLY</u> recommended that you delete the \"install\" directory once SimpleRisk has been confirmed to be up and running.</b><br /><br />\n";
                }
        }
        else
        {
                echo "I couldn't find a config.php file located at " . $file . ".  The contents that the config.php file needs to contain are printed out below so you can update it yourself.<br /><br />\n";
                echo "<b>For security reasons, it is <u>HIGHLY</u> recommended that you delete the \"install\" directory once SimpleRisk has been confirmed to be up and running.</b><br /><br />\n";
        }

	echo "</form>\n";
        echo "<hr><br />\n";
	echo "<form name=\"display_config\" method=\"post\" action=\"\" class=\"loginForm\">\n";
        echo nl2br(htmlentities($config_file, ENT_QUOTES, 'utf-8'));
	echo "</form>\n";
}

/*************************************
 * FUNCTION: DATABASE ALREADY EXISTS *
 *************************************/
function database_already_exists()
{
	global $escaper;

        // Database Connection Information
        $dbhost = addslashes($_POST['host']);
        $dbport = addslashes($_POST['port']);
        $dbuser = addslashes($_POST['user']);
        $dbpass = addslashes($_POST['pass']);

        // Remove any backticks from DB connection information
        $pattern = '/`/';
        $replacement = '';
        $dbhost = preg_replace($pattern, $replacement, $dbhost);
        $dbport = preg_replace($pattern, $replacement, $dbport);
        $dbuser = preg_replace($pattern, $replacement, $dbuser);
        $dbpass = preg_replace($pattern, $replacement, $dbpass);

        // Connect to the mysql database
        $db = installer_db_open($dbhost, $dbport, $dbuser, $dbpass, "mysql");

        // SimpleRisk Installation Information
        $sr_host = addslashes($_POST['sr_host']);
        $sr_db = addslashes($_POST['sr_db']);
        $sr_user = addslashes($_POST['sr_user']);
        $sr_pass = addslashes($_POST['sr_pass']);

        // Remove any backticks from DB connection information
        $pattern = '/`/';
        $replacement = '';
        $sr_host = preg_replace($pattern, $replacement, $sr_host);
        $sr_db = preg_replace($pattern, $replacement, $sr_db);
        $sr_user = preg_replace($pattern, $replacement, $sr_user);
        $sr_pass = preg_replace($pattern, $replacement, $sr_pass);

	// Check if the database already exists
        $stmt = $db->prepare("SHOW DATABASES LIKE :sr_db");
	$stmt->bindParam(":sr_db", $sr_db, PDO::PARAM_STR);
        $stmt->execute();
	$array = $stmt->fetchAll();

	// If the array is not empty
	if (!empty($array))
	{
		return "A database with the name \"" . $escaper->escapeHtml($sr_db) . "\" already exists.";
	}

	// Check if the username already exists in the user table
	$stmt = $db->prepare("SELECT * FROM user WHERE User=:sr_user");
	$stmt->bindParam(":sr_user", $sr_user, PDO::PARAM_STR);
	$stmt->execute();
	$array = $stmt->fetchAll();

	// If the array is not empty
	if (!empty($array))
	{
		return "An entry in the mysql user table with the name \"" . $escaper->escapeHtml($sr_user) . "\" already exists.";
	}

        // Check if the username already exists in the db table
        $stmt = $db->prepare("SELECT * FROM db WHERE User=:sr_user");
        $stmt->bindParam(":sr_user", $sr_user, PDO::PARAM_STR);
        $stmt->execute();
        $array = $stmt->fetchAll();

        // If the array is not empty
        if (!empty($array))
        {
                return "An entry in the mysql db table with the name \"" . $escaper->escapeHtml($sr_user) . "\" already exists.";
        }

	return false;
}

/********************************************
 * FUNCTION: INSTALLER UPDATE CONFIGURATION *
 ********************************************/
function installer_update_configuration()
{
        echo "<h1 class=\"text-center welcome--msg\">SimpleRisk Installer - Installation Complete</h1>\n";
        echo "<form name=\"install_complete\" method=\"post\" action=\"\" class=\"loginForm\">\n";

        // Path to the file in the tmp dir
        $tmp_dir = sys_get_temp_dir();
        $tmp_config_file = realpath($tmp_dir . '/config.php');

        // This should be the path to the config.php file
        $file = realpath(__DIR__ . '/../includes/config.php');

        // If we successfully copy the file to the right location
        if (copy($tmp_config_file, $file))
        {
                // Delete the temporary file
                unlink($tmp_config_file);

                echo "Configuration file has been updated successfully.<br /><br />\n";
                echo "SimpleRisk should now be communicating with the database.<br /><br />\n";
                echo "<b>For security reasons, it is <u>HIGHLY</u> recommended that you delete the \"install\" directory once SimpleRisk has been confirmed to be up and running.</b><br /><br />\n";
        }
        else
        {
                echo "Something happened and we weren't able to place the configuration file at " . $file . ".  Sorry.<br /><br />\n";
                echo "<b>For security reasons, it is <u>HIGHLY</u> recommended that you delete the \"install\" directory once SimpleRisk has been confirmed to be up and running.</b><br /><br />\n";
        }

	echo "</form>\n";
}

/*******************************************************
 * FUNCTION: CHECK INSTALLER APPLICATION VERSION MATCH *
 *******************************************************/
function check_installer_application_version_match($release_version)
{
	$current_app_version = current_version("app");

	// If the SimpleRisk application matches the installer
	if ($current_app_version === $release_version)
	{
		health_check_good("The SimpleRisk database installer version (" . $release_version . ") matches the SimpleRisk application version (" . $current_app_version . ").");
	}
	else
	{
		health_check_bad("The SimpleRisk database install version (" . $release_version . ") does not match the installed SimpleRisk application version (" . $current_app_version . ").");
	}
}

/*****************************************
 * FUNCTION: CHECK INSTALLER PHP VERSION *
 *****************************************/
function check_installer_php_version()
{
	// Get the version of PHP
	if (!defined('PHP_VERSION_ID'))
	{
		$version = explode('.', PHP_VERSION);
		define('PHP_VERSION_ID', ($version[0] * 10000 + $version[1] * 100 + $version[2]));
	}

	// Check if PHP >= 5.5 for trend report
	if (PHP_VERSION_ID >= 50500)
	{
		// If PHP is not version 7
		if (PHP_VERSION_ID < 70000)
		{
			health_check_good("Detected PHP >= 5.5 which should function properly for SimpleRisk.");
			return true;
		}
		else
		{
			health_check_bad("SimpleRisk does not currently work with PHP >= 7.0.");
			return false;
		}
	}
	else
	{
		health_check_bad("The SimpleRisk trend report requires PHP >= 5.5 to function properly.");
		return false;
	}
}

/********************************
 * FUNCTION: CHECK MYSQL STRICT *
 ********************************/
function check_mysql_strict()
{
        // Database Connection Information
        $dbhost = addslashes($_POST['host']);
        $dbport = addslashes($_POST['port']);
        $dbuser = addslashes($_POST['user']);
        $dbpass = addslashes($_POST['pass']);

        // Remove any backticks from DB connection information
        $pattern = '/`/';
        $replacement = '';
        $dbhost = preg_replace($pattern, $replacement, $dbhost);
        $dbport = preg_replace($pattern, $replacement, $dbport);
        $dbuser = preg_replace($pattern, $replacement, $dbuser);
        $dbpass = preg_replace($pattern, $replacement, $dbpass);

        // Connect to the mysql database
        $db = installer_db_open($dbhost, $dbport, $dbuser, $dbpass, "mysql");

	// Query for the current SQL mode
	$stmt = $db->prepare("SELECT @@sql_mode;");
        $stmt->execute();
	$array = $stmt->fetch();
	$sql_mode = $array['@@sql_mode'];

        // Close the mysql database
        installer_db_close($db);

	// If the row contains STRICT_TRANS_TABLES
	if (preg_match("/.*STRICT_TRANS_TABLES.*/", $sql_mode))
	{
		return true;
	}
	else
	{
		return false;
	}
}

/**************************************
 * FUNCTION: CHECK MYSQL NO ZERO DATE *
 **************************************/
function check_mysql_no_zero_date()
{
        // Database Connection Information
        $dbhost = addslashes($_POST['host']);
        $dbport = addslashes($_POST['port']);
        $dbuser = addslashes($_POST['user']);
        $dbpass = addslashes($_POST['pass']);

        // Remove any backticks from DB connection information
        $pattern = '/`/';
        $replacement = '';
        $dbhost = preg_replace($pattern, $replacement, $dbhost);
        $dbport = preg_replace($pattern, $replacement, $dbport);
        $dbuser = preg_replace($pattern, $replacement, $dbuser);
        $dbpass = preg_replace($pattern, $replacement, $dbpass);

        // Connect to the mysql database
        $db = installer_db_open($dbhost, $dbport, $dbuser, $dbpass, "mysql");

        // Query for the current SQL mode
        $stmt = $db->prepare("SELECT @@sql_mode;");
        $stmt->execute();
        $array = $stmt->fetch();
        $sql_mode = $array['@@sql_mode'];

        // Close the mysql database
        installer_db_close($db);

        // If the row contains NO_ZERO_DATE 
        if (preg_match("/.*NO_ZERO_DATE.*/", $sql_mode))
        {
                return true;
        }
        else
        {
                return false;
        }
}

/********************************************
 * FUNCTION: CHECK MYSQL ONLY FULL GROUP BY *
 ********************************************/
function check_mysql_only_full_group_by()
{       
        // Database Connection Information
        $dbhost = addslashes($_POST['host']);
        $dbport = addslashes($_POST['port']);
        $dbuser = addslashes($_POST['user']);
        $dbpass = addslashes($_POST['pass']);

        // Remove any backticks from DB connection information
        $pattern = '/`/';
        $replacement = '';
        $dbhost = preg_replace($pattern, $replacement, $dbhost);
        $dbport = preg_replace($pattern, $replacement, $dbport);
        $dbuser = preg_replace($pattern, $replacement, $dbuser);
        $dbpass = preg_replace($pattern, $replacement, $dbpass);
        
        // Connect to the mysql database 
        $db = installer_db_open($dbhost, $dbport, $dbuser, $dbpass, "mysql");
        
        // Query for the current SQL mode
        $stmt = $db->prepare("SELECT @@sql_mode;");
        $stmt->execute();
        $array = $stmt->fetch();
        $sql_mode = $array['@@sql_mode'];
        
        // Close the mysql database
        installer_db_close($db);
        
        // If the row contains ONLY_FULL_GROUP_BY 
        if (preg_match("/.*ONLY_FULL_GROUP_BY.*/", $sql_mode))
        {       
                return true;
        }
        else    
        {       
                return false;
        }
}

/****************************
 * FUNCTION: GENERATE TOKEN *
 ****************************/
function generate_token($size)
{
        $token = "";
        $values = array_merge(range(0, 9), range('a', 'z'), range('A', 'Z'));
	$values_count = count($values);

        for ($i = 0; $i < $size; $i++)
        {
		// The the random int function exists (PHP 7)
		if (function_exists('random_int'))
		{
			// Generate the token using the random_int function
			$token .= $values[random_int(0, $values_count-1)];
		}
                else $token .= $values[array_rand($values)];
        }

        return $token;
}

/***********************
 * FUNCTION: LOAD FILE *
 ***********************/
function load_file($dbhost, $dbport, $dbuser, $dbpass, $sr_db, $file)
{
	// Create the file path
	$path_to = realpath(__DIR__ . '/db/');
	$file_path = addslashes($path_to . '/' . $file);

	// Temporary variable to store current query
	$templine = "";

	// Read in entire file
	$lines = file($file_path);

        // Connect to the simplerisk database
        $db = installer_db_open($dbhost, $dbport, $dbuser, $dbpass, $sr_db);

	// For each line in the file
	foreach ($lines as $line)
	{
		// Comment pattern
		$pattern = "(#[^\r\n]+|/\*.*?\*/|//[^\r\n]+|--.*[\r\n])";

		// If the line is a comment
		if (preg_match($pattern, $line))
		{
			// Skip it
			continue;
		}

		// Add this line to the current segment
		$templine .= $line;

		// If it has a semicolon at the end, it's the end of the query
		if (substr(trim($line), -1, 1) == ';')
		{
			// Perform the query
                	$stmt = $db->prepare($templine);
                	try
			{
                        	$stmt->execute();

				// Reset the temporary variable to empty
				$templine = "";
                	}
			catch (PDOException $e)
			{
                        	echo 'Schema load failed: ' . $e->getMessage();
                        	installer_db_close($db);
                        	return false;
                	}
		}
	}

	// Close the simplerisk database
        installer_db_close($db);

	echo "Database Schema Loaded Successfully!<br /><br />\n";

	return true;
}

/********************************
 * FUNCTION: CREATE CONFIG FILE *
 ********************************/
function create_config_file($dbhost, $dbport, $sr_user, $sr_pass, $sr_db, $schema, $db_sessions, $db_ssl_cert_path)
{
	// Set the default language based on the selected schema
	switch ($schema)
	{
		case 1:
			$lang = "en";
			break;
		case 2:
			$lang = "es";
			break;
		case 3:
			$lang = "bp";
			break;
		default:
			$lang = "es";
			break;
	}

	$content = "<?php\n";
	$content .= " /* This Source Code Form is subject to the terms of the Mozilla Public\n";
	$content .= " * License, v. 2.0. If a copy of the MPL was not distributed with this\n";
	$content .= " * file, You can obtain one at http://mozilla.org/MPL/2.0/. */\n";
	$content .= "\n";
	$content .= "// MySQL Database Host Name\n";
	$content .= "define('DB_HOSTNAME', '" . $dbhost . "');\n";
	$content .= "\n";
	$content .= "// MySQL Database Port Number\n";
	$content .= "define('DB_PORT', '" . $dbport . "');\n";
	$content .= "\n";
	$content .= "// MySQL Database User Name\n";
	$content .= "define('DB_USERNAME', '" . $sr_user . "');\n";
	$content .= "\n";
	$content .= "// MySQL Database Password\n";
	$content .= "define('DB_PASSWORD', '" . $sr_pass . "');\n";
	$content .= "\n";
	$content .= "// MySQL Database Name\n";
	$content .= "define('DB_DATABASE', '" . $sr_db . "');\n";
	$content .= "\n";
	$content .= "// Use database for sessions\n";
	$content .= "define('USE_DATABASE_FOR_SESSIONS', '" . $db_sessions . "');\n";
	$content .= "\n";
	$content .= "// Path to the certificate to be used for SSL connections to the database\n";
	$content .= "define('DB_SSL_CERTIFICATE_PATH', '" . $db_ssl_cert_path . "');\n";
	$content .= "\n";
	$content .= "?>";

	return $content;
}

/*******************************
 * FUNCTION: TIMEZONE DROPDOWN *
 *******************************/
function timezone_dropdown()
{
	$zones = timezone_identifiers_list();
       
	echo "<select name=\"timezone\">\n";

        // If a timezone value was posted
        if (isset($_POST) && isset($_POST['timezone']))
	{
		// Get the selected timezone
		$selected = $_POST['timezone'];

		// List all timezones with the selected as the default
                foreach ($zones as $zone)
                {
                        echo "<option value=\"" . $zone . "\"" . ($zone == $selected ? " selected" : "") . ">" . $zone . "</option>\n";
                }
	}
	else
	{
		// List all timezones with America/Chicago as the default
		foreach ($zones as $zone)
		{
			echo "<option value=\"" . $zone . "\"" . ($zone == "America/Chicago" ? " selected" : "") . ">" . $zone . "</option>\n";
    		}
	}

	echo "</select>\n";
}

/*****************************
 * FUNCTION: GET CURRENT URL *
 *****************************/
function get_current_url()
{
	// Check if we are using the HTTPS protocol
	$isHTTPS = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == "on");

	// Set the port
	$port = (isset($_SERVER['SERVER_PORT']) && ((!$isHTTPS && $_SERVER['SERVER_PORT'] != "80") || ($isHTTPS && $_SERVER['SERVER_PORT'] != "443")));
	$port = ($port) ? ":" . $_SERVER['SERVER_PORT'] : "";

	// Set the current URL
	$url = ($isHTTPS ? "https://" : "http://") . $_SERVER['SERVER_NAME'] . $port . $_SERVER['REQUEST_URI'];

	// Strip the installer path from the URL
	$url = preg_replace('/\/install\/$/', '', $url);

	// Return the URL
	return $url;
}

?>
